from typing import Any
from unittest.mock import Mock


class Storage(dict):
    def __getattr__(self, key): ...
    def __setattr__(self, key, value): ...
    def __delattr__(self, key): ...
    def __repr__(self): ...
    def __sub__(self, other): ...
    @staticmethod
    def of(object) -> 'Storage': ...


class ChainMock:
    def __init__(self, path: str, return_value) -> None: ...
    def join(self, path, return_value) -> 'ChainMock': ...
    def __call__(self, path=None) -> Any: ...
