from typing import *


class AnySub:
    def __init__(self, core: Type) -> None: ...


def generic_origin(t): ...
def generic_args(t): ...
def optional_core(t): ...
def collection_core(t: Type): ...
def issubtyping(cls: Type, parent: Type) -> bool: ...
